﻿using FFmpeg.AutoGen;
using FFmpeg.AutoGen.Example;
using OpenCvSharp;
using System;
using System.IO;

namespace WindowsFormsApp1.FFmpeg
{
    public class FFmpegVideoWriter : IDisposable
    {
        double _fps;
        public double Fps => _fps;

        OpenCvSharp.Size _frameSize;
        public OpenCvSharp.Size FrameSize => _frameSize;

        FileStream _outputStream;
        H264VideoStreamEncoder _streamEncoder;
        VideoFrameConverter _vfc;

        int _frameNumber = 0;

        public FFmpegVideoWriter(string filePath, FourCC type, double fps,
                OpenCvSharp.Size frameSize, bool color)
        {
            _frameSize = frameSize;
            _fps = fps;

            System.Drawing.Size imageSize = new System.Drawing.Size(frameSize.Width, frameSize.Height);
            AVPixelFormat sourcePixelFormat = AVPixelFormat.AV_PIX_FMT_BGR24;
            AVPixelFormat destinationPixelFormat = AVPixelFormat.AV_PIX_FMT_YUV420P;

            // destinationPixelFormat = AVPixelFormat.AV_PIX_FMT_YUVJ422P;

            _outputStream = File.Open(filePath, FileMode.Create);
            _streamEncoder = new H264VideoStreamEncoder(_outputStream, AVCodecID.AV_CODEC_ID_H264, destinationPixelFormat, (int)fps, imageSize);

            _vfc = new VideoFrameConverter(imageSize, sourcePixelFormat, imageSize, destinationPixelFormat);
        }

        public unsafe void Write(Mat matFrame)
        {
            byte* pBitmapData = (byte*)matFrame.Data.ToPointer();

            var data = new byte_ptrArray8 { [0] = pBitmapData };
            var linesize = new int_array8 { [0] = (matFrame.ElemSize() * matFrame.Width) };

            var frame = new AVFrame
            {
                data = data,
                linesize = linesize,
                height = matFrame.Height
            };

            var convertedFrame = _vfc.Convert(frame);
            convertedFrame.pts = _frameNumber * (int)_fps;

            _streamEncoder.Encode(convertedFrame);
            _frameNumber++;
        }

        public void Dispose()
        {
            _streamEncoder.Dispose();
            _outputStream.Dispose();
            _vfc.Dispose();
        }
    }
}
